@@ -166,7 +166,7 @@ SQLRETURN SQLBulkOperations(
                 ERROR_HY010, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     if ( statement -> state == STATE_S4 )
@@ -181,7 +181,7 @@ SQLRETURN SQLBulkOperations(
                 ERROR_24000, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     if ( statement -> state == STATE_S7 )
@@ -196,7 +196,7 @@ SQLRETURN SQLBulkOperations(
                 ERROR_HY010, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     if ( statement -> state == STATE_S8 ||
@@ -216,7 +216,7 @@ SQLRETURN SQLBulkOperations(
                 ERROR_HY010, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     if ( statement -> state == STATE_S11 ||
@@ -234,10 +234,14 @@ SQLRETURN SQLBulkOperations(
                     ERROR_HY010, NULL,
                     statement -> connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
     }
 
+    if ( statement -> state != STATE_S11 && statement -> state != STATE_S12 )
+    {
+        statement -> interupted_state = statement -> state;
+    }
     /*
      * there are a lot of conditions that should be tested here
      */
@@ -270,7 +274,7 @@ SQLRETURN SQLBulkOperations(
                 ERROR_IM001, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     if ( ret == SQL_STILL_EXECUTING )
@@ -286,6 +290,10 @@ SQLRETURN SQLBulkOperations(
         statement -> interupted_state = statement -> state;
         statement -> state = STATE_S8;
     }
+    else
+    {
+        statement -> state = statement -> interupted_state;
+    }
 
     if ( log_info.log_flag )
     {
@@ -300,5 +308,5 @@ SQLRETURN SQLBulkOperations(
                 statement -> msg );
     }
 
-    return function_return( SQL_HANDLE_STMT, statement, ret );
+    return function_return( SQL_HANDLE_STMT, statement, ret, DEFER_R3 );
 }
