@@ -190,7 +190,12 @@ SQLRETURN SQLDataSources( SQLHENV environment_handle,
 
     if ( log_info.log_flag )
     {
-        sprintf( environment -> msg, "\n\t\tEntry:\
+#ifdef HAVE_SNPRINTF
+            snprintf( environment -> msg, sizeof(  environment -> msg ),
+#else
+            sprintf( environment -> msg, 
+#endif
+                "\n\t\tEntry:\
 \n\t\t\tEnvironment = %p",
                 environment );
 
@@ -387,7 +392,11 @@ SQLRETURN SQLDataSources( SQLHENV environment_handle,
 
     if ( log_info.log_flag )
     {
+#ifdef HAVE_SNPRINTF
+        snprintf( environment -> msg, sizeof(  environment -> msg ),
+#else
         sprintf( environment -> msg, 
+#endif
                 "\n\t\tExit:[%s]",
                     __get_return_status( SQL_SUCCESS, s1 ));
 
