@@ -317,6 +317,7 @@ static char const rcsid[]= "$RCSfile: SQLDriverConnect.c,v $ $Revision: 1.28 $";
  */
 
 extern int pooling_enabled;
+extern int pool_wait_timeout;
 
 void __generate_connection_string( struct con_struct *con_str, char *str, int str_len )
 {
@@ -363,6 +364,7 @@ char *tmp;
 
         if ( strlen( str ) + strlen( tmp ) > str_len )
         {
+            free( tmp );
             break;
         }
         else
@@ -463,6 +465,13 @@ struct con_pair * con_p;
     if ( keyword )
     {
         con_p = malloc( sizeof( *con_p ));
+        if ( !con_p )
+        {
+            free(keyword);
+            if ( value )
+                free(value);
+            return NULL;
+        }
         con_p -> keyword = keyword;
         con_p -> attribute = value;
         return con_p;
@@ -695,6 +704,7 @@ SQLRETURN SQLDriverConnect(
     SQLCHAR local_out_conection[ 2048 ];
     char *save_filedsn;
     int warnings = 0;
+    CPOOLHEAD *pooh = 0;
 
     /*
      * check connection
@@ -892,93 +902,144 @@ SQLRETURN SQLDriverConnect(
 
     connection -> pooled_connection = NULL;
 
-    if ( pooling_enabled && search_for_pool( connection, 
-                                                NULL, 0,
-                                                NULL, 0,
-                                                NULL, 0,
-                                                conn_str_in, len_conn_str_in ))
+    if ( pooling_enabled )
     {
+        int retpool;
+        int retrying = 0;
+        time_t wait_begin = time( NULL );
+
+retry:
+        retpool = search_for_pool(  connection,
+                                NULL, 0,
+                                NULL, 0,
+                                NULL, 0,
+                                conn_str_in, len_conn_str_in, &pooh, retrying );
         /*
-         * copy the in string to the out string
+         * found usable existing connection from pool
          */
-
-        ret_from_connect = SQL_SUCCESS;
-
-        if ( conn_str_out )
+        if ( retpool == 1 )
         {
-            if ( len_conn_str_in < 0 )
+            /*
+             * copy the in string to the out string
+             */
+
+            ret_from_connect = SQL_SUCCESS;
+
+            if ( conn_str_out )
             {
-                len_conn_str_in = strlen((char*) conn_str_in );
+                if ( len_conn_str_in < 0 )
+                {
+                    len_conn_str_in = strlen((char*) conn_str_in );
+                }
+
+                if ( len_conn_str_in >= conn_str_out_max )
+                {
+                    memcpy( conn_str_out, conn_str_in, conn_str_out_max - 1 );
+                    conn_str_out[ conn_str_out_max - 1 ] = '\0';
+                    if ( ptr_conn_str_out )
+                    {
+                        *ptr_conn_str_out = len_conn_str_in;
+                    }
+
+                    __post_internal_error( &connection -> error,
+                        ERROR_01004, NULL,
+                        connection -> environment -> requested_version );
+
+                    ret_from_connect = SQL_SUCCESS_WITH_INFO;
+                }
+                else
+                {
+                    memcpy( conn_str_out, conn_str_in, len_conn_str_in );
+                    conn_str_out[ len_conn_str_in ] = '\0';
+                    if ( ptr_conn_str_out )
+                    {
+                        *ptr_conn_str_out = len_conn_str_in;
+                    }
+                }
             }
 
-            if ( len_conn_str_in >= conn_str_out_max )
+            if ( log_info.log_flag )
             {
-                memcpy( conn_str_out, conn_str_in, conn_str_out_max - 1 );
-                conn_str_out[ conn_str_out_max - 1 ] = '\0';
-                if ( ptr_conn_str_out )
-                {
-                    *ptr_conn_str_out = len_conn_str_in;
-                }
+                sprintf( connection -> msg,
+                        "\n\t\tExit:[%s]",
+                            __get_return_status( ret_from_connect, s1 ));
+
+                dm_log_write( __FILE__,
+                            __LINE__,
+                        LOG_INFO,
+                        LOG_INFO,
+                        connection -> msg );
+            }
+
+            connection -> state = STATE_C4;
+
+            __release_conn( &con_struct );
+
+            return function_return( SQL_HANDLE_DBC, connection, ret_from_connect, DEFER_R0 );
+        }
+
+        /*
+         * pool is at capacity
+         */
+        if ( retpool == 2 )
+        {
+            /*
+             * either no timeout or exceeded the timeout
+             */
+            if ( ! pool_wait_timeout || time( NULL ) - wait_begin > pool_wait_timeout )
+            {
+                mutex_pool_exit();
+                dm_log_write( __FILE__,
+                    __LINE__,
+                    LOG_INFO,
+                    LOG_INFO,
+                    "Error: HYT02" );
 
                 __post_internal_error( &connection -> error,
-                    ERROR_01004, NULL,
+                    ERROR_HYT02, NULL,
                     connection -> environment -> requested_version );
 
-                ret_from_connect = SQL_SUCCESS_WITH_INFO;
-            }
-            else
-            {
-                memcpy( conn_str_out, conn_str_in, len_conn_str_in );
-                conn_str_out[ len_conn_str_in ] = '\0';
-                if ( ptr_conn_str_out )
-                {
-                    *ptr_conn_str_out = len_conn_str_in;
-                }
+                __release_conn( &con_struct );
+
+                return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
             }
+
+            /*
+             * wait up to 1 second for a signal and try again
+             */
+            pool_timedwait( connection );
+            retrying = 1;
+            goto retry;
         }
 
-        if ( log_info.log_flag )
-        {
-            sprintf( connection -> msg,
-                    "\n\t\tExit:[%s]",
-                        __get_return_status( ret_from_connect, s1 ));
-
-            dm_log_write( __FILE__,
-                        __LINE__,
-                    LOG_INFO,
-                    LOG_INFO,
-                    connection -> msg );
-        }
-
-        connection -> state = STATE_C4;
-
-        __release_conn( &con_struct );
-
-        return function_return( SQL_HANDLE_DBC, connection, ret_from_connect, DEFER_R0 );
+        /*
+         * 1 pool entry has been reserved. Early exits henceforth need to unreserve.
+         */
     }
 
     /*
-     * else safe the info for later
+     * else save the info for later
      */
 
     if ( pooling_enabled )
     {
         connection -> dsn_length = 0;
 
-        strcpy( connection -> server, "" );
+        connection -> _server = strdup( "" );
         connection -> server_length = 0;
-        strcpy( connection -> user, "" );
+        connection -> _user = strdup( "" );
         connection -> user_length = 0;
-        strcpy( connection -> password, "" );
+        connection -> _password = strdup( "" );
         connection -> password_length = 0;
 
         if ( len_conn_str_in == SQL_NTS )
         {
-            strcpy( connection -> driver_connect_string, (char*)conn_str_in );
+            connection -> _driver_connect_string = strdup((char*)conn_str_in );
         }
         else
         {
-            memcpy( connection -> driver_connect_string, conn_str_in, len_conn_str_in );
+            connection -> _driver_connect_string = calloc( len_conn_str_in, 1 );
+            memcpy( connection -> _driver_connect_string, conn_str_in, len_conn_str_in );
         }
         connection -> dsn_length = len_conn_str_in;
     }
@@ -1066,23 +1127,30 @@ SQLRETURN SQLDriverConnect(
                                 if ( save_filedsn ) {
                                     free( save_filedsn );
                                 }
+
+                                pool_unreserve( pooh );
                         
+                                __release_conn( &con_struct );
+                                __release_conn( &con_struct1 );
+
                                 return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
                             }
 
-							if ( strlen((char*) conn_str_in ) > 0 )
-							{
-                				sprintf( str1, ";%s=%s", cp -> keyword, cp -> attribute );
-							}
-							else
-							{
-                   				sprintf( str1, "%s=%s", cp -> keyword, cp -> attribute );
-							}
-
-                            if ( strlen( (char*)conn_str_in ) + strlen( str1 ) < conn_str_out_max ) {
-                			    strcat((char*) conn_str_in, str1 );
+                            if ( strlen((char*) conn_str_in ) > 0 )
+                            {
+                                sprintf( str1, ";%s=%s", cp -> keyword, cp -> attribute );
                             }
-                            else {
+                            else
+                            {
+                                sprintf( str1, "%s=%s", cp -> keyword, cp -> attribute );
+                            }
+
+                            if ( strlen( (char*)conn_str_in ) + strlen( str1 ) < conn_str_out_max )
+                            {
+                                strcat((char*) conn_str_in, str1 );
+                            }
+                            else
+                            {
                                 warnings = 1;
                                 __post_internal_error( &connection -> error,
                                         ERROR_01004, NULL,
@@ -1090,7 +1158,7 @@ SQLRETURN SQLDriverConnect(
                             }
 
                             free( str1 );
-						}
+                        }
 
                         cp = cp -> next;
                     }
@@ -1122,16 +1190,20 @@ SQLRETURN SQLDriverConnect(
                                 if ( save_filedsn ) {
                                     free( save_filedsn );
                                 }
-                        
+
+                                pool_unreserve( pooh );
+
+                                __release_conn( &con_struct1 );
+
                                 return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
                             }
 
-							if ( strlen((char*) conn_str_in ) > 0 )
-							{
+                            if ( strlen((char*) conn_str_in ) > 0 )
+                            {
                             	sprintf( str1, ";%s=%s", cp -> keyword, cp -> attribute );
-							}
-							else
-							{
+                            }
+                            else
+                            {
                             	sprintf( str1, "%s=%s", cp -> keyword, cp -> attribute );
 							}
                             if ( strlen( (char*)conn_str_in ) + strlen( str1 ) < conn_str_out_max ) {
@@ -1199,7 +1271,11 @@ SQLRETURN SQLDriverConnect(
             if ( save_filedsn ) {
                 free( save_filedsn );
             }
-                        
+
+            pool_unreserve( pooh );
+
+            __release_conn( &con_struct );
+
             return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
         }
 
@@ -1251,6 +1327,8 @@ SQLRETURN SQLDriverConnect(
                 free( save_filedsn );
             }
 
+            pool_unreserve( pooh );
+
             return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
         }
 
@@ -1272,6 +1350,8 @@ SQLRETURN SQLDriverConnect(
                 free( save_filedsn );
             }
 
+            pool_unreserve( pooh );
+
             return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
         }
 
@@ -1297,6 +1377,8 @@ SQLRETURN SQLDriverConnect(
                 free( save_filedsn );
             }
 
+            pool_unreserve( pooh );
+
             return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
         }
 
@@ -1333,6 +1415,8 @@ SQLRETURN SQLDriverConnect(
 
         __disconnect_part_four( connection );       /* release unicode handles */
 
+        pool_unreserve( pooh );
+
         return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
     }
 
@@ -1355,6 +1439,8 @@ SQLRETURN SQLDriverConnect(
             free( save_filedsn );
         }
 
+        pool_unreserve( pooh );
+
         return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
     }
 
@@ -1485,9 +1571,11 @@ SQLRETURN SQLDriverConnect(
                 free( save_filedsn );
             }
 
+            pool_unreserve( pooh );
+
             return function_return( SQL_HANDLE_DBC, connection, ret_from_connect, DEFER_R0 );
         }
-		connection -> unicode_driver = 0;
+        connection -> unicode_driver = 0;
     }
     else
     {
@@ -1549,7 +1637,7 @@ SQLRETURN SQLDriverConnect(
                             sqlstate,
                             &native_error,
                             message_text,
-                            sizeof( message_text ),
+                            sizeof( message_text ) / sizeof( SQLWCHAR ),
                             &ind );
 
 
@@ -1590,7 +1678,7 @@ SQLRETURN SQLDriverConnect(
                             sqlstate,
                             &native_error,
                             message_text,
-                            sizeof( message_text ),
+                            sizeof( message_text ) / sizeof( SQLWCHAR ),
                             &ind );
 
 
@@ -1648,6 +1736,8 @@ SQLRETURN SQLDriverConnect(
                 free( s1 );
             }
 
+            pool_unreserve( pooh );
+
             return function_return( SQL_HANDLE_DBC, connection, ret_from_connect, DEFER_R0 );
         }
         else
@@ -1698,6 +1788,8 @@ SQLRETURN SQLDriverConnect(
             free( save_filedsn );
         }
 
+        pool_unreserve( pooh );
+
         return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R0 );
     }
 
@@ -1840,5 +1932,10 @@ SQLRETURN SQLDriverConnect(
         ret_from_connect = SQL_SUCCESS_WITH_INFO;
     }
 
+    if ( pooling_enabled && !add_to_pool( connection, pooh ) )
+    {
+        pool_unreserve( pooh );
+    }
+
     return function_return_nodrv( SQL_HANDLE_DBC, connection, ret_from_connect );
 }
