@@ -189,7 +189,7 @@ SQLRETURN SQLExtendedFetch(
                 ERROR_HY106, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     /*
@@ -210,7 +210,7 @@ SQLRETURN SQLExtendedFetch(
                 ERROR_HY010, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     if ( statement -> state == STATE_S4 )
@@ -225,7 +225,7 @@ SQLRETURN SQLExtendedFetch(
                 ERROR_24000, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     if ( statement -> state == STATE_S6 )
@@ -240,7 +240,7 @@ SQLRETURN SQLExtendedFetch(
                 ERROR_HY010, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     if ( statement -> state == STATE_S8 ||
@@ -260,7 +260,7 @@ SQLRETURN SQLExtendedFetch(
                 ERROR_HY010, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     if ( statement -> state == STATE_S11 ||
@@ -278,7 +278,7 @@ SQLRETURN SQLExtendedFetch(
                     ERROR_HY010, NULL,
                     statement -> connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
     }
 
@@ -294,7 +294,7 @@ SQLRETURN SQLExtendedFetch(
                 ERROR_IM001, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     ret = SQLEXTENDEDFETCH( statement -> connection,
@@ -309,16 +309,25 @@ SQLRETURN SQLExtendedFetch(
         statement -> interupted_func = SQL_API_SQLEXTENDEDFETCH;
         if ( statement -> state != STATE_S11 &&
                 statement -> state != STATE_S12 )
+        {
+            statement -> interupted_state = statement -> state;
             statement -> state = STATE_S11;
+        }
     }
     else if ( SQL_SUCCEEDED( ret ))
     {
+        statement -> eod = 0;
         statement -> state = STATE_S7;
     }
     else if ( ret == SQL_NO_DATA )
     {
+        statement -> eod = 1;
         statement -> state = STATE_S7;
     }
+    else if (statement -> state == STATE_S11 || statement -> state == STATE_S12 )
+    {
+        statement -> state = statement -> interupted_state;
+    }
 
     if ( log_info.log_flag )
     {
@@ -333,5 +342,5 @@ SQLRETURN SQLExtendedFetch(
                 statement -> msg );
     }
 
-    return function_return( SQL_HANDLE_STMT, statement, ret );
+    return function_return( SQL_HANDLE_STMT, statement, ret, DEFER_R3 );
 }
