@@ -110,14 +110,7 @@ SQLRETURN SQLGetConnectOptionW( SQLHDBC connection_handle,
     {
         if ( value )
         {
-            if ( log_info.log_flag )
-            {
-                *((SQLINTEGER*)value) = SQL_OPT_TRACE_ON;
-            }
-            else
-            {
-                *((SQLINTEGER*)value) = SQL_OPT_TRACE_ON;
-            }
+            *((SQLINTEGER*)value) = SQL_OPT_TRACE_ON;
         }
 
         return SQL_SUCCESS;
@@ -215,7 +208,7 @@ SQLRETURN SQLGetConnectOptionW( SQLHDBC connection_handle,
                 ERROR_HY010, NULL,
                 connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     if ( connection -> state == STATE_C2 )
@@ -224,6 +217,8 @@ SQLRETURN SQLGetConnectOptionW( SQLHDBC connection_handle,
         {
           case SQL_ACCESS_MODE:
           case SQL_AUTOCOMMIT:
+          case SQL_LOGIN_TIMEOUT:
+          case SQL_ODBC_CURSORS:
             break;
 
           default:
@@ -237,7 +232,7 @@ SQLRETURN SQLGetConnectOptionW( SQLHDBC connection_handle,
                     ERROR_08003, NULL,
                     connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
     }
 
@@ -314,7 +309,7 @@ SQLRETURN SQLGetConnectOptionW( SQLHDBC connection_handle,
                 LOG_INFO, 
                 connection -> msg );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_SUCCESS );
+        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_SUCCESS );
     }
     else
     {
@@ -383,7 +378,7 @@ SQLRETURN SQLGetConnectOptionW( SQLHDBC connection_handle,
                         ERROR_IM001, NULL,
                         connection -> environment -> requested_version );
 
-                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
             }
         }
         else
@@ -487,7 +482,7 @@ SQLRETURN SQLGetConnectOptionW( SQLHDBC connection_handle,
                         ERROR_IM001, NULL,
                         connection -> environment -> requested_version );
 
-                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
             }
         }
 
@@ -504,6 +499,6 @@ SQLRETURN SQLGetConnectOptionW( SQLHDBC connection_handle,
                     connection -> msg );
         }
 
-        return function_return( SQL_HANDLE_DBC, connection, ret );
+        return function_return( SQL_HANDLE_DBC, connection, ret, DEFER_R3 );
     }
 }
