@@ -215,6 +215,7 @@ static int is_char_diag( int diag_identifier )
     switch( diag_identifier ) {
         case SQL_DIAG_CLASS_ORIGIN:
         case SQL_DIAG_CONNECTION_NAME:
+        case SQL_DIAG_DYNAMIC_FUNCTION:
         case SQL_DIAG_MESSAGE_TEXT:
         case SQL_DIAG_SERVER_NAME:
         case SQL_DIAG_SQLSTATE:
@@ -235,6 +236,11 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
 {
     ERROR *ptr;
 
+    if ( is_char_diag( diag_identifier ) && buffer_length < 0 )
+    {
+        return SQL_ERROR;
+    }
+    
     /*
      * check the header fields first
      */
@@ -247,7 +253,7 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
             SQLLEN val;
             SQLRETURN ret;
 
-            if ( head -> handle_type != SQL_HANDLE_STMT )
+            if ( rec_number > 0 || head -> handle_type != SQL_HANDLE_STMT )
             {
                 return SQL_ERROR;
             }
@@ -325,15 +331,27 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
         {
             SQLRETURN ret;
 
-            if ( head -> handle_type != SQL_HANDLE_STMT )
+            if ( rec_number > 0 )
             {
                 return SQL_ERROR;
             }
+            else if ( head -> handle_type != SQL_HANDLE_STMT )
+            {
+                if ( diag_info_ptr )
+                {
+                    strcpy( diag_info_ptr, "" );
+                }
+                if ( string_length_ptr )
+                {
+                    *string_length_ptr = 0;
+                }
+                return SQL_SUCCESS;
+            }
             else if ( head -> header_set )
             {
                 if ( SQL_SUCCEEDED( head -> diag_dynamic_function_ret ) && diag_info_ptr )
                 {
-                    unicode_to_ansi_copy( diag_info_ptr, buffer_length, head -> diag_dynamic_function, buffer_length, __get_connection( head ), NULL );
+                    unicode_to_ansi_copy( diag_info_ptr, buffer_length, head -> diag_dynamic_function, SQL_NTS, __get_connection( head ), NULL );
                     if ( string_length_ptr )
                     {
                         *string_length_ptr = wide_strlen( head -> diag_dynamic_function );
@@ -357,7 +375,7 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
                         0,
                         diag_identifier,
                         s1 ? s1 : diag_info_ptr,
-                        buffer_length,
+                        buffer_length * sizeof ( SQLWCHAR ),
                         string_length_ptr );
 
                 if ( SQL_SUCCEEDED( ret ) && diag_info_ptr && s1 )
@@ -365,6 +383,11 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
                     unicode_to_ansi_copy( diag_info_ptr, buffer_length, s1, buffer_length, __get_connection( head ), NULL );
                 }
 
+                if ( string_length_ptr && *string_length_ptr > 0 )
+                {
+                    *string_length_ptr /= sizeof ( SQLWCHAR );
+                }
+
                 if ( s1 )
                 {
                     free( s1 );
@@ -398,10 +421,15 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
             SQLINTEGER val;
             SQLRETURN ret;
 
-            if ( head -> handle_type != SQL_HANDLE_STMT )
+            if ( rec_number > 0  )
             {
                 return SQL_ERROR;
             }
+            else if ( head -> handle_type != SQL_HANDLE_STMT )
+            {
+                *((SQLINTEGER*)diag_info_ptr) = 0;
+                return SQL_SUCCESS;
+            }
             else if ( head -> header_set )
             {
                 if ( SQL_SUCCEEDED( head -> diag_dynamic_function_code_ret ) && diag_info_ptr )
@@ -454,6 +482,10 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
         {
             SQLINTEGER val;
             
+            if ( rec_number > 0 )
+            {
+                return SQL_ERROR;
+            }
             val = head -> sql_diag_head.internal_count + 
                 head -> sql_diag_head.error_count;
 
@@ -479,7 +511,9 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
      * else check the records
      */
 
-    if ( rec_number < 1 )
+    if ( rec_number < 1 ||
+        (( diag_identifier == SQL_DIAG_COLUMN_NUMBER ||
+          diag_identifier == SQL_DIAG_ROW_NUMBER ) && head -> handle_type != SQL_HANDLE_STMT ))
     {
         return SQL_ERROR;
     }
@@ -510,11 +544,10 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
         {
             SQLRETURN ret;
             SQLWCHAR *s1 = NULL;
-			int char_buffer_len = sizeof( SQLWCHAR ) * buffer_length;
 
-            if ( is_char_diag( diag_identifier ) && diag_info_ptr && buffer_length > 0 )
+            if ( diag_info_ptr && buffer_length > 0 )
             {
-                s1 = malloc( char_buffer_len + sizeof( SQLWCHAR ));
+                s1 = malloc( ( buffer_length + 1 ) * sizeof( SQLWCHAR ));
             }
 
             ret = SQLGETDIAGFIELDW( __get_connection( head ),
@@ -523,12 +556,17 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
                     rec_number,
                     diag_identifier,
                     s1 ? s1 : diag_info_ptr,
-                    s1 ? char_buffer_len + sizeof( SQLWCHAR ) : buffer_length,
+                    s1 ? ( buffer_length + 1 ) * sizeof( SQLWCHAR ) : buffer_length,
                     string_length_ptr );
 
             if ( SQL_SUCCEEDED( ret ) && s1 && diag_info_ptr )
             {
                 unicode_to_ansi_copy( diag_info_ptr, buffer_length, s1, SQL_NTS, __get_connection( head ), NULL );
+
+                if ( string_length_ptr && *string_length_ptr > 0 ) 
+                {
+                    *string_length_ptr /= sizeof( SQLWCHAR );
+                }
             }
 
             if ( s1 )
@@ -536,21 +574,13 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
                 free( s1 );
             }
 
-			if ( string_length_ptr && *string_length_ptr > 0 ) 
-			{
-				*string_length_ptr /= sizeof( SQLWCHAR );
-			}
-
             if ( SQL_SUCCEEDED( ret ) && diag_identifier == SQL_DIAG_SQLSTATE )
             {
                 /*
                  * map 3 to 2 if required
                  */
                 if ( diag_info_ptr )
-                {
-                    if ( diag_info_ptr )
-                        __map_error_state( diag_info_ptr, __get_version( head ));
-                }
+                    __map_error_state( diag_info_ptr, __get_version( head ));
             }
 
             return ret;
@@ -575,10 +605,7 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
                  * map 3 to 2 if required
                  */
                 if ( diag_info_ptr )
-                {
-                    if ( diag_info_ptr )
-                        __map_error_state( diag_info_ptr, __get_version( head ));
-                }
+                    __map_error_state( diag_info_ptr, __get_version( head ));
             }
 
             return ret;
@@ -613,7 +640,7 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
         {
             if ( SQL_SUCCEEDED( ptr -> diag_class_origin_ret ))
             {
-                unicode_to_ansi_copy( diag_info_ptr, buffer_length, ptr -> diag_class_origin, buffer_length, __get_connection( head ), NULL );
+                unicode_to_ansi_copy( diag_info_ptr, buffer_length, ptr -> diag_class_origin, SQL_NTS, __get_connection( head ), NULL );
                 if ( string_length_ptr )
                 {
                     *string_length_ptr = wide_strlen( ptr -> diag_class_origin );
@@ -641,7 +668,7 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
         {
             if ( SQL_SUCCEEDED( ptr -> diag_connection_name_ret ))
             {
-                unicode_to_ansi_copy( diag_info_ptr, buffer_length, ptr -> diag_connection_name, buffer_length, __get_connection( head ), NULL );
+                unicode_to_ansi_copy( diag_info_ptr, buffer_length, ptr -> diag_connection_name, SQL_NTS, __get_connection( head ), NULL );
                 if ( string_length_ptr )
                 {
                     *string_length_ptr = wide_strlen( ptr -> diag_connection_name );
@@ -713,7 +740,7 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
         {
             if ( SQL_SUCCEEDED( ptr -> diag_server_name_ret ))
             {
-                unicode_to_ansi_copy( diag_info_ptr, buffer_length, ptr -> diag_server_name, buffer_length, __get_connection( head ), NULL );
+                unicode_to_ansi_copy( diag_info_ptr, buffer_length, ptr -> diag_server_name, SQL_NTS, __get_connection( head ), NULL );
                 if ( string_length_ptr )
                 {
                     *string_length_ptr = wide_strlen( ptr -> diag_server_name );
@@ -772,7 +799,7 @@ static SQLRETURN extract_sql_error_field( EHEAD *head,
         {
             if ( SQL_SUCCEEDED( ptr -> diag_subclass_origin_ret ))
             {
-                unicode_to_ansi_copy( diag_info_ptr, buffer_length, ptr -> diag_subclass_origin, buffer_length, __get_connection( head ), NULL );
+                unicode_to_ansi_copy( diag_info_ptr, buffer_length, ptr -> diag_subclass_origin, SQL_NTS, __get_connection( head ), NULL );
                 if ( string_length_ptr )
                 {
                     *string_length_ptr = wide_strlen( ptr -> diag_subclass_origin );
@@ -818,248 +845,151 @@ SQLRETURN SQLGetDiagField( SQLSMALLINT handle_type,
     SQLRETURN ret;
     SQLCHAR s1[ 100 + LOG_MESSAGE_LEN ];
 
-    if ( handle_type == SQL_HANDLE_ENV )
+    DMHENV environment = ( DMHENV ) handle;
+    DMHDBC connection = NULL;
+    DMHSTMT statement = NULL;
+    DMHDESC descriptor = NULL;
+
+    EHEAD *herror;
+    char *handle_msg;
+    const char *handle_type_ptr;
+
+    switch ( handle_type )
     {
-        DMHENV environment = ( DMHENV ) handle;
+        case SQL_HANDLE_ENV:
+            {
+                if ( !__validate_env( environment ))
+                {
+                    dm_log_write( __FILE__,
+                            __LINE__,
+                            LOG_INFO,
+                            LOG_INFO,
+                            "Error: SQL_INVALID_HANDLE" );
 
-        if ( !__validate_env( environment ))
-        {
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    "Error: SQL_INVALID_HANDLE" );
+                    return SQL_INVALID_HANDLE;
+                }
 
-            return SQL_INVALID_HANDLE;
-        }
+                herror = &environment->error;
+                handle_msg = environment->msg;
+                handle_type_ptr = "Environment";
+            }
+            break;
 
-        thread_protect( SQL_HANDLE_ENV, environment );
+        case SQL_HANDLE_DBC:
+            {
+                connection = ( DMHDBC ) handle;
 
-        if ( log_info.log_flag )
-        {
-            sprintf( environment -> msg, 
-                "\n\t\tEntry:\
-\n\t\t\tEnvironment = %p\
+                if ( !__validate_dbc( connection ))
+                {
+                    return SQL_INVALID_HANDLE;
+                }
+
+                herror = &connection->error;
+                handle_msg = connection->msg;
+                handle_type_ptr = "Connection";
+            }
+            break;
+
+        case SQL_HANDLE_STMT:
+            {
+                statement = ( DMHSTMT ) handle;
+
+                if ( !__validate_stmt( statement ))
+                {
+                    return SQL_INVALID_HANDLE;
+                }
+
+                connection = statement->connection;
+                herror = &statement->error;
+                handle_msg = statement->msg;
+                handle_type_ptr = "Statement";
+            }
+            break;
+
+        case SQL_HANDLE_DESC:
+            {
+                descriptor = ( DMHDESC ) handle;
+
+                if ( !__validate_desc( descriptor ))
+                {
+                    return SQL_INVALID_HANDLE;
+                }
+
+                connection = descriptor->connection;
+                herror = &descriptor->error;
+                handle_msg = descriptor->msg;
+                handle_type_ptr = "Descriptor";
+            }
+            break;
+
+        default:
+            {
+                return SQL_NO_DATA;
+            }
+    }
+
+    thread_protect( handle_type, handle );
+
+    if ( log_info.log_flag )
+    {
+        sprintf( handle_msg,
+            "\n\t\tEntry:\
+\n\t\t\t%s = %p\
 \n\t\t\tRec Number = %d\
 \n\t\t\tDiag Ident = %d\
 \n\t\t\tDiag Info Ptr = %p\
 \n\t\t\tBuffer Length = %d\
 \n\t\t\tString Len Ptr = %p",
-                    environment,
-                    rec_number,
-                    diag_identifier,
-                    diag_info_ptr,
-                    buffer_length,
-                    string_length_ptr );
-
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    environment -> msg );
-        }
-
-        ret = extract_sql_error_field( &environment -> error,
+                handle_type_ptr,
+                handle,
                 rec_number,
                 diag_identifier,
                 diag_info_ptr,
                 buffer_length,
                 string_length_ptr );
 
-        if ( log_info.log_flag )
-        {
-            sprintf( environment -> msg, 
-                    "\n\t\tExit:[%s]",
-                        __get_return_status( ret, s1 ));
-
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    environment -> msg );
-        }
-
-        thread_release( SQL_HANDLE_ENV, environment );
-
-        return ret;
+        dm_log_write( __FILE__,
+                __LINE__,
+                LOG_INFO,
+                LOG_INFO,
+                handle_msg );
     }
-    else if ( handle_type == SQL_HANDLE_DBC )
+
+    /*
+     * Do diag extraction here if defer flag is set.
+     * Clean the flag after extraction.
+     */
+    if ( connection && herror->defer_extract )
     {
-        DMHDBC connection = ( DMHDBC ) handle;
+        extract_error_from_driver( herror, connection, herror->ret_code_deferred, 0 );
 
-        if ( !__validate_dbc( connection ))
-        {
-            return SQL_INVALID_HANDLE;
-        }
-
-        thread_protect( SQL_HANDLE_DBC, connection );
-
-        if ( log_info.log_flag )
-        {
-            sprintf( connection -> msg, 
-                "\n\t\tEntry:\
-\n\t\t\tConnection = %p\
-\n\t\t\tRec Number = %d\
-\n\t\t\tDiag Ident = %d\
-\n\t\t\tDiag Info Ptr = %p\
-\n\t\t\tBuffer Length = %d\
-\n\t\t\tString Len Ptr = %p",
-                    connection,
-                    rec_number,
-                    diag_identifier,
-                    diag_info_ptr,
-                    buffer_length,
-                    string_length_ptr );
-
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    connection -> msg );
-        }
-
-        ret = extract_sql_error_field( &connection -> error,
-                rec_number,
-                diag_identifier,
-                diag_info_ptr,
-                buffer_length,
-                string_length_ptr );
-
-        if ( log_info.log_flag )
-        {
-            sprintf( connection -> msg, 
-                    "\n\t\tExit:[%s]",
-                        __get_return_status( ret, s1 ));
-
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    connection -> msg );
-        }
-
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return ret;
+        herror->defer_extract = 0;
+        herror->ret_code_deferred = 0;
     }
-    else if ( handle_type == SQL_HANDLE_STMT )
+
+    ret = extract_sql_error_field( herror,
+            rec_number,
+            diag_identifier,
+            diag_info_ptr,
+            buffer_length,
+            string_length_ptr );
+
+    if ( log_info.log_flag )
     {
-        DMHSTMT statement = ( DMHSTMT ) handle;
+        sprintf( handle_msg,
+                "\n\t\tExit:[%s]",
+                __get_return_status( ret, s1 ));
 
-        if ( !__validate_stmt( statement ))
-        {
-            return SQL_INVALID_HANDLE;
-        }
-
-        thread_protect( SQL_HANDLE_STMT, statement );
-
-        if ( log_info.log_flag )
-        {
-            sprintf( statement -> msg, 
-                "\n\t\tEntry:\
-\n\t\t\tStatement = %p\
-\n\t\t\tRec Number = %d\
-\n\t\t\tDiag Ident = %d\
-\n\t\t\tDiag Info Ptr = %p\
-\n\t\t\tBuffer Length = %d\
-\n\t\t\tString Len Ptr = %p",
-                    statement,
-                    rec_number,
-                    diag_identifier,
-                    diag_info_ptr,
-                    buffer_length,
-                    string_length_ptr );
-
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    statement -> msg );
-        }
-
-        ret = extract_sql_error_field( &statement -> error,
-                rec_number,
-                diag_identifier,
-                diag_info_ptr,
-                buffer_length,
-                string_length_ptr );
-
-        if ( log_info.log_flag )
-        {
-            sprintf( statement -> msg, 
-                    "\n\t\tExit:[%s]",
-                        __get_return_status( ret, s1 ));
-
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    statement -> msg );
-        }
-
-        thread_release( SQL_HANDLE_STMT, statement );
-
-        return ret;
+        dm_log_write( __FILE__,
+                __LINE__,
+                LOG_INFO,
+                LOG_INFO,
+                handle_msg );
     }
-    else if ( handle_type == SQL_HANDLE_DESC )
-    {
-        DMHDESC descriptor = ( DMHDESC ) handle;
 
-        if ( !__validate_desc( descriptor ))
-        {
-            return SQL_INVALID_HANDLE;
-        }
+    thread_release( handle_type, handle );
 
-        thread_protect( SQL_HANDLE_DESC, descriptor );
+    return ret;
 
-        if ( log_info.log_flag )
-        {
-            sprintf( descriptor -> msg, 
-                "\n\t\tEntry:\
-\n\t\t\tDescriptor = %p\
-\n\t\t\tRec Number = %d\
-\n\t\t\tDiag Ident = %d\
-\n\t\t\tDiag Info Ptr = %p\
-\n\t\t\tBuffer Length = %d\
-\n\t\t\tString Len Ptr = %p",
-                    descriptor,
-                    rec_number,
-                    diag_identifier,
-                    diag_info_ptr,
-                    buffer_length,
-                    string_length_ptr );
-
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    descriptor -> msg );
-        }
-
-        ret = extract_sql_error_field( &descriptor -> error,
-                rec_number,
-                diag_identifier,
-                diag_info_ptr,
-                buffer_length,
-                string_length_ptr );
-
-        if ( log_info.log_flag )
-        {
-            sprintf( descriptor -> msg, 
-                    "\n\t\tExit:[%s]",
-                        __get_return_status( ret, s1 ));
-
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    descriptor -> msg );
-        }
-
-        thread_release( SQL_HANDLE_DESC, descriptor );
-
-        return ret;
-    }
-    return SQL_NO_DATA;
 }
 
