@@ -118,7 +118,7 @@ SQLRETURN SQLGetFunctions( SQLHDBC connection_handle,
            SQLUSMALLINT *supported )
 {
     DMHDBC connection = (DMHDBC)connection_handle;
-    SQLCHAR s1[ 100 + LOG_MESSAGE_LEN ];
+    SQLCHAR s0[ 24 ], s1[ 100 + LOG_MESSAGE_LEN ];
 
     /*
      * check connection
@@ -202,7 +202,7 @@ SQLRETURN SQLGetFunctions( SQLHDBC connection_handle,
         sprintf( connection -> msg, 
                 "\n\t\tExit:[%s]\
 \n\t\t\tSupported = %s",
-                    __get_return_status( SQL_SUCCESS, s1 ),
+                    __get_return_status( SQL_SUCCESS, s0 ),
                     __sptr_as_string( s1, (short*)supported ));
 
         dm_log_write( __FILE__, 
