@@ -248,12 +248,13 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                         LOG_INFO, 
                         LOG_INFO, 
                         "Error: HY024" );
-        
+
+                function_entry( connection );
                 __post_internal_error( &connection -> error,
                     ERROR_HY024, NULL,
                     connection -> environment -> requested_version );
         
-                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
             }
             else 
             {
@@ -317,12 +318,13 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                             LOG_INFO, 
                             LOG_INFO, 
                             "Error: HY024" );
-            
+
+                    function_entry( connection );
                     __post_internal_error( &connection -> error,
                         ERROR_HY024, NULL,
                         connection -> environment -> requested_version );
             
-                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                    return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
                 }
                 else 
                 {
@@ -347,13 +349,14 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                         __LINE__, 
                         LOG_INFO, 
                         LOG_INFO, 
-                        "Error: HY009" );
-        
+                        "Error: HY024" );
+
+                function_entry( connection );
                 __post_internal_error( &connection -> error,
-                    ERROR_HY009, NULL,
+                    ERROR_HY024, NULL,
                     connection -> environment -> requested_version );
         
-                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
             }
             else 
             {
@@ -416,7 +419,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                 ERROR_08003, NULL,
                 connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
     }
     else if ( connection -> state == STATE_C3 )
@@ -431,7 +434,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
             ERROR_HY010, NULL,
             connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
     else if ( connection -> state == STATE_C4 ||
             connection -> state == STATE_C5 ||
@@ -449,7 +452,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                 ERROR_08002, NULL,
                 connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
         else if ( attribute == SQL_ATTR_PACKET_SIZE )
         {
@@ -463,7 +466,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                 ERROR_HY011, NULL,
                 connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
     }
 
@@ -484,7 +487,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                 ERROR_HY024, NULL,
                 connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     /*
@@ -493,10 +496,34 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
 
     switch( attribute ) 
     {
+        /* ODBC 3.x statement attributes are not settable at the connection level */
+        case SQL_ATTR_APP_PARAM_DESC:
+        case SQL_ATTR_APP_ROW_DESC:
+        case SQL_ATTR_CURSOR_SCROLLABLE:
+        case SQL_ATTR_CURSOR_SENSITIVITY:
+        case SQL_ATTR_ENABLE_AUTO_IPD:
+        case SQL_ATTR_FETCH_BOOKMARK_PTR:
+        case SQL_ATTR_IMP_PARAM_DESC:
+        case SQL_ATTR_IMP_ROW_DESC:
+        case SQL_ATTR_PARAM_BIND_OFFSET_PTR:
+        case SQL_ATTR_PARAM_BIND_TYPE:
+        case SQL_ATTR_PARAM_OPERATION_PTR:
+        case SQL_ATTR_PARAM_STATUS_PTR:
+        case SQL_ATTR_PARAMS_PROCESSED_PTR:
+        case SQL_ATTR_PARAMSET_SIZE:
+        case SQL_ATTR_ROW_ARRAY_SIZE:
+        case SQL_ATTR_ROW_BIND_OFFSET_PTR:
+        case SQL_ATTR_ROW_OPERATION_PTR:
+        case SQL_ATTR_ROW_STATUS_PTR:
+        case SQL_ATTR_ROWS_FETCHED_PTR:
+            __post_internal_error( &connection -> error,
+                    ERROR_HY092, NULL,
+                    connection -> environment -> requested_version );
+
+            return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
+
       	case SQL_ATTR_CONCURRENCY:
 	  	case SQL_BIND_TYPE:
-      	case SQL_ATTR_CURSOR_SCROLLABLE:
-      	case SQL_ATTR_CURSOR_SENSITIVITY:
       	case SQL_ATTR_CURSOR_TYPE:
       	case SQL_ATTR_MAX_LENGTH:
       	case SQL_MAX_ROWS:
@@ -519,28 +546,11 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                         ERROR_24000, NULL,
                         connection -> environment -> requested_version );
 
-                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
             }
             break;
 
         default:
-            if ( attribute == SQL_ATTR_CURRENT_CATALOG ) 
-            {
-                if( __check_stmt_from_dbc_v( connection, 3, STATE_S5, STATE_S6, STATE_S7 )) {
-
-                    dm_log_write( __FILE__, 
-                            __LINE__, 
-                            LOG_INFO, 
-                            LOG_INFO, 
-                            "Error: 24000" );
-
-                    __post_internal_error( &connection -> error,
-                            ERROR_24000, NULL,
-                            connection -> environment -> requested_version );
-
-                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
-                }
-            }
 
             if( __check_stmt_from_dbc_v( connection, 5, STATE_S8, STATE_S9, STATE_S10, STATE_S11, STATE_S12 )) {
 
@@ -554,7 +564,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                         ERROR_HY010, NULL,
                         connection -> environment -> requested_version );
 
-                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
             }
             break;
     }
@@ -637,30 +647,65 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
         else if ( attribute != SQL_ATTR_LOGIN_TIMEOUT )
         {
             /*
-             * save any unknown attributes untill connect
+             * save any unknown attributes until connect
              */
 
-            struct save_attr *sa = calloc( 1, sizeof( struct save_attr ));
+            struct save_attr sa, *sap;
 
-            sa -> attr_type = attribute;
+            memset( &sa, 0, sizeof ( sa ));
+
+            sa.attr_type = attribute;
             if ( string_length > 0 )
             {
-                sa -> str_attr = malloc( string_length );
-                memcpy( sa -> str_attr, value, string_length );
-                sa -> str_len = string_length;
+                sa.str_attr = malloc( string_length );
+                memcpy( sa.str_attr, value, string_length );
+                sa.str_len = string_length;
             }
             else if ( string_length == SQL_NTS )
             {
-                sa -> str_attr = strdup( value );
-                sa -> str_len = string_length;
+                if (!value)
+                {
+                    __post_internal_error( &connection -> error,
+                        ERROR_HY024, "Invalid argument value",
+                        connection -> environment -> requested_version );
+                    return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                }
+                else
+                {
+                    sa.str_attr = strdup( value );
+                    sa.str_len = string_length;
+                }
             }
             else
             {
-                sa -> int_attr = ( SQLLEN ) value;
-                sa -> str_len = string_length;
+                sa.intptr_attr = (intptr_t) value;
+                sa.str_len = string_length;
+            }
+            
+            sap = connection -> save_attr;
+            
+            while ( sap )
+            {
+                if ( sap -> attr_type == attribute )
+                {
+                    free ( sap -> str_attr );
+                    break;
+                }
+                sap = sap -> next;
+            }
+            
+            if ( sap ) /* replace existing attribute */
+            {
+                *sap = sa;
+            }
+            else
+            {
+                sap = malloc( sizeof( struct save_attr ));
+                *sap = sa;
+
+                sap -> next = connection -> save_attr;
+                connection -> save_attr = sap;
             }
-            sa -> next = connection -> save_attr;
-            connection -> save_attr = sa;
         }
 
         sprintf( connection -> msg, 
@@ -673,7 +718,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                 LOG_INFO, 
                 connection -> msg );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_SUCCESS );
+        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_SUCCESS );
     }
     else
     {
@@ -700,13 +745,13 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                                 ERROR_HY092, NULL,
                                 connection -> environment -> requested_version );
 
-                        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
                     }
 
                     ret = SQLSETCONNECTOPTION( connection,
                             connection -> driver_dbc,
                             attribute,
-                            value );
+                            (SQLULEN) value );
                 }
                 else
                 {
@@ -720,7 +765,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                             ERROR_IM001, NULL,
                             connection -> environment -> requested_version );
 
-                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                    return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
                 }
             }
             else
@@ -757,7 +802,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                                 ERROR_HY092, NULL,
                                 connection -> environment -> requested_version );
 
-                        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
                     }
 
                     switch( attribute )
@@ -770,7 +815,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                         ret = SQLSETCONNECTOPTIONW( connection,
                                 connection -> driver_dbc,
                                 attribute,
-                                s1 );
+                                (SQLULEN) s1 );
 
                         if ( s1 )
                             free( s1 );
@@ -781,7 +826,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                         ret = SQLSETCONNECTOPTIONW( connection,
                                 connection -> driver_dbc,
                                 attribute,
-                                value );
+                                (SQLULEN) value );
                         break;
                     }
                 }
@@ -797,7 +842,7 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
                             ERROR_IM001, NULL,
                             connection -> environment -> requested_version );
 
-                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                    return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
                 }
             }
             else
@@ -855,5 +900,5 @@ SQLRETURN SQLSetConnectAttr( SQLHDBC connection_handle,
         connection -> bookmarks_on = (SQLLEN) value;
     }
 
-    return function_return( SQL_HANDLE_DBC, connection, ret );
+    return function_return( SQL_HANDLE_DBC, connection, ret, DEFER_R3 );
 }
