@@ -905,10 +905,18 @@ void __release_dbc( DMHDBC connection )
         }
     }
 
-    if ( connection -> _driver_connect_string ) 
-    {
+    if ( connection -> _driver_connect_string ) {
         free( connection -> _driver_connect_string );
     }
+    if ( connection -> _user ) {
+        free( connection -> _user );
+    }
+    if ( connection -> _password ) {
+        free( connection -> _password );
+    }
+    if ( connection -> _server ) {
+        free( connection -> _server );
+    }
 
     /*
      * clear just to make sure
@@ -1904,7 +1912,7 @@ void thread_release( int type, void *handle )
 
 int pool_timedwait( DMHDBC connection )
 {
-    int ret;
+    int ret = 0;
     struct timespec waituntil;
 
 #ifdef HAVE_CLOCK_GETTIME
