@@ -16,6 +16,9 @@
 #ifdef HAVE_TIME_H
 #include <time.h>
 #endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h> 
+#endif
 
 #ifdef HAVE_SYNCH_H
 #include <synch.h>
@@ -384,11 +387,11 @@ typedef struct connection
     int             ttl;
     char            *_driver_connect_string;
     int             dsn_length;
-    char            server[ 128 ];
+    char            *_server;
     int             server_length;
-    char            user[ 128 ];
+    char            *_user;
     int             user_length;
-    char            password[ 128 ];
+    char            *_password;
     int             password_length;
     char            cli_year[ 5 ];
     struct attr_struct  env_attribute;      /* Extended attribute set info */
@@ -428,11 +431,11 @@ typedef struct connection_pool_head
 
     char    *_driver_connect_string;
     int     dsn_length;
-    char    server[ 128 ];
+    char    *_server;
     int     server_length;
-    char    user[ 128 ];
+    char    *_user;
     int     user_length;
-    char    password[ 128 ];
+    char    *_password;
     int     password_length;
 
     volatile int num_entries;                /* always at least 1 */
@@ -1173,7 +1176,7 @@ int add_to_pool( DMHDBC connection, CPOOLHEAD *pooh );
                                         (stmt,pn,dtp,psp,ddp,np)
 
 #define DM_SQLDISCONNECT            21
-#define CHECK_SQLDISCONNECT(con)    (con->functions[21].func!=NULL)
+#define CHECK_SQLDISCONNECT(con)    (con->functions!=NULL && con->functions[21].func!=NULL)
 #define SQLDISCONNECT(con,dbc)\
                                     ((SQLRETURN (*)(SQLHDBC))\
                                     con->functions[21].func)(dbc)
