@@ -15,6 +15,10 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "tuple.h"
 #include <string.h>
 #include <stdlib.h>
@@ -49,7 +53,7 @@ void set_tuplefield_int4(TupleField *tuple_field, Int4 value)
 {
 char buffer[15];
 
-	sprintf(buffer,"%ld", value);
+	sprintf(buffer,"%ld", (long)value);
 
 	tuple_field->len = strlen(buffer)+1;
 	/* +1 ... is this correct (better be on the save side-...) */
