@@ -10,16 +10,9 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "driver.h"
 
-/* HERES THE CORRECT WAY...
-SQLRETURN SQLDescribeParam( SQLHSTMT      hDrvStmt,
-                                  SQLUSMALLINT  nParmNumber,
-                                  SQLSMALLINT   *pnDataType,
-                                  SQLUINTEGER   *pnSize,
-                                  SQLSMALLINT   *pnDecDigits,
-                                  SQLSMALLINT   *pnNullable )
-HERES THE WAY THAT WORKS... */
 SQLRETURN SQLDescribeParam( SQLHSTMT      hDrvStmt,
                                   SQLUSMALLINT  nParmNumber,
                                   SQLSMALLINT   *pnDataType,
@@ -33,7 +26,7 @@ SQLRETURN SQLDescribeParam( SQLHSTMT      hDrvStmt,
     if( NULL == hStmt )
         return SQL_INVALID_HANDLE;
 
-	sprintf((char*) hStmt->szSqlMsg, "hStmt = $%08lX", hStmt );
+	sprintf((char*) hStmt->szSqlMsg, "hStmt = $%08lX", (long)hStmt );
     logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_WARNING, LOG_WARNING,(char*) hStmt->szSqlMsg );
 
     /************************
