@@ -3,7 +3,7 @@
  * unixODBC Cursor Library
  *
  * Created by Nick Gorham
- * (nick@easysoft.com).
+ * (nick@lurcher.org).
  *
  * copyright (c) 1999 Nick Gorham
  *
@@ -23,9 +23,12 @@
  *
  **********************************************************************
  *
- * $Id: SQLExecDirect.c,v 1.5 2007/11/13 15:04:57 lurcher Exp $
+ * $Id: SQLExecDirect.c,v 1.6 2009/02/18 17:59:17 lurcher Exp $
  *
  * $Log: SQLExecDirect.c,v $
+ * Revision 1.6  2009/02/18 17:59:17  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
  * Revision 1.5  2007/11/13 15:04:57  lurcher
  * Fix 64 bit cursor lib issues
  *
@@ -60,6 +63,7 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "cursorlibrary.h"
 
 /*
@@ -203,15 +207,23 @@ SQLRETURN get_column_names( CLHSTMT cl_statement )
 
     cl_statement -> column_names = malloc( sizeof(char *) 
             * cl_statement -> column_count );
+    if ( !cl_statement->column_names )
+        return SQL_ERROR;
 
     cl_statement -> data_type = malloc( sizeof( SQLSMALLINT ) 
             * cl_statement -> column_count );
+    if ( !cl_statement->data_type )
+        return SQL_ERROR;
 
     cl_statement -> column_size = malloc( sizeof( SQLULEN ) 
             * cl_statement -> column_count );
+    if ( !cl_statement->column_size )
+        return SQL_ERROR;
 
     cl_statement -> decimal_digits = malloc( sizeof( SQLSMALLINT ) 
             * cl_statement -> column_count );
+    if ( !cl_statement->decimal_digits )
+        return SQL_ERROR;
 
     for ( i = 1; i <= cl_statement -> column_count; i ++ )
     {
@@ -230,7 +242,7 @@ SQLRETURN get_column_names( CLHSTMT cl_statement )
         ret = SQLDESCRIBECOL( cl_statement -> cl_connection,
                 cl_statement -> driver_stmt,
                 i,
-                cname, 
+                (SQLCHAR*) cname,
                 sizeof( cname ),
                 NULL,
                 &cl_statement -> data_type[ i - 1 ], 
