@@ -3,7 +3,7 @@
  * unixODBC Cursor Library
  *
  * Created by Nick Gorham
- * (nick@easysoft.com).
+ * (nick@lurcher.org).
  *
  * copyright (c) 1999 Nick Gorham
  *
@@ -23,9 +23,15 @@
  *
  **********************************************************************
  *
- * $Id: SQLGetStmtOption.c,v 1.2 2005/10/27 17:54:49 lurcher Exp $
+ * $Id: SQLGetStmtOption.c,v 1.4 2009/02/18 17:59:17 lurcher Exp $
  *
  * $Log: SQLGetStmtOption.c,v $
+ * Revision 1.4  2009/02/18 17:59:17  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
+ * Revision 1.3  2009/02/17 09:47:45  lurcher
+ * Clear up a number of bugs
+ *
  * Revision 1.2  2005/10/27 17:54:49  lurcher
  * fix what I suspect is a typo in qt.m4
  *
@@ -45,6 +51,7 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "cursorlibrary.h"
 
 SQLRETURN CLGetStmtOption( SQLHSTMT statement_handle,
@@ -100,7 +107,7 @@ SQLRETURN CLGetStmtOption( SQLHSTMT statement_handle,
         break;
 
       case SQL_ATTR_ROWS_FETCHED_PTR:
-        *(( SQLUINTEGER ** ) value ) = cl_statement -> rows_fetched_ptr;
+        *(( SQLULEN ** ) value ) = cl_statement -> rows_fetched_ptr;
         break;
 
       case SQL_ATTR_USE_BOOKMARKS:
