@@ -59,11 +59,18 @@
 
 /* XXX Hmm... Perhaps autoconf this up again? */
 
+#include <config.h>
+
 #define HAVE_LONG_DOUBLE
 
+#ifdef HAVE_STDDEF_H
+#include <stddef.h>
+#endif
 #include <string.h>
 #include <ctype.h>
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
 #include "uodbc_extras.h"
 
 /* varargs declarations: */
@@ -342,7 +349,11 @@ static int dopr (char *buffer, size_t maxlen, const char *format, va_list args)
 	break;
       case 'p':
 	strvalue = (char*) va_arg (args, void *);
+#ifdef HAVE_PTRDIFF_T
+	fmtint (buffer, &currlen, maxlen, (ptrdiff_t) strvalue, 16, min, max, flags);
+#else
 	fmtint (buffer, &currlen, maxlen, (long) strvalue, 16, min, max, flags);
+#endif
 	break;
       case 'n':
 	if (cflags == DP_C_SHORT) 
@@ -533,7 +544,7 @@ static LDOUBLE abs_val (LDOUBLE value)
   return result;
 }
 
-static LDOUBLE pow10 (int exponent)
+static LDOUBLE local_pow10 (int exponent)
 {
   LDOUBLE result = 1;
 
@@ -546,7 +557,7 @@ static LDOUBLE pow10 (int exponent)
   return result;
 }
 
-static long round (LDOUBLE value)
+static long int_round (LDOUBLE value)
 {
   long intpart;
 
@@ -607,12 +618,12 @@ static void fmtfp (char *buffer, size_t *currlen, size_t maxlen,
   /* We "cheat" by converting the fractional part to integer by
    * multiplying by a factor of 10
    */
-  fracpart = round ((pow10 (max)) * (ufvalue - intpart));
+  fracpart = int_round ((local_pow10 (max)) * (ufvalue - intpart));
 
-  if (fracpart >= pow10 (max))
+  if (fracpart >= local_pow10 (max))
   {
     intpart++;
-    fracpart -= pow10 (max);
+    fracpart -= local_pow10 (max);
   }
 
 #ifdef DEBUG_SNPRINTF
