@@ -7,10 +7,10 @@
  * binary compatability between drivers developed under different
  * packages).
  *
- * You can include this file directly but it is almost always
- * included indirectly, by including.. for example sqlext.h
+ * You can include this file directly, but it is almost always
+ * included indirectly, by including, for example sqlext.h
  *
- * This include makes no effort to be usefull on any platforms other
+ * This include makes no effort to be useful on any platforms other
  * than Linux (with some exceptions for UNIX in general).
  *
  * !!!DO NOT CONTAMINATE THIS FILE WITH NON-Linux CODE!!!
@@ -20,14 +20,14 @@
 #define __SQLTYPES_H
 
 /****************************
- * default to the 3.51 definitions. should define ODBCVER before here if you want an older set of defines
+ * default to the 3.80 definitions. should define ODBCVER before here if you want an older set of defines
  ***************************/
 #ifndef ODBCVER
-#define ODBCVER	0x0351
+#define ODBCVER	0x0380
 #endif
 
 /*
- * if thi sis set, then use a 4 byte unicode definition, insteead of the 2 bye that MS use
+ * if this is set, then use a 4 byte unicode definition, instead of the 2 byte definition that MS use
  */
 
 #ifdef SQL_WCHART_CONVERT  
@@ -44,22 +44,26 @@ extern "C" {
 #endif
 
 /*
- * this is defined by configure, but will not be on a normal application build
- * the install creates a unixodbc_conf.h file that contains the current build settings
+ * unixODBC needs to know the size of a long integer to #define certain data types.
+ * SIZEOF_LONG_INT is defined by unixODBC but is not usually defined by other programs.
+ * In these cases, the compiler uses #defines stored in unixodbc.h to determine the
+ * correct data types for the target system.
+ *
+ * Refer to unixodbc_conf.h for other build-time settings.
  */
 
 #ifndef SIZEOF_LONG_INT
-#include <unixodbc_conf.h>
+#include "unixodbc.h"
 #endif
 
 #ifndef SIZEOF_LONG_INT
-#error "Needs to know how big a long int is to continue!!!"
+#error "unixODBC needs to know the size of a `long int' to continue!!!"
 #endif
 
 /****************************
  * These make up for having no windows.h
  ***************************/
-#ifndef ALLREADY_HAVE_WINDOWS_TYPE
+#ifndef ALREADY_HAVE_WINDOWS_TYPE
 
 #define FAR
 #define CALLBACK
@@ -69,30 +73,28 @@ extern "C" {
 #define SQL_API
 #endif
 #define	BOOL				int
+#ifndef _WINDOWS_
 typedef void*				HWND;
+#endif
 typedef char				CHAR;
 #ifdef UNICODE
 
 /* 
  * NOTE: The Microsoft unicode define is only for apps that want to use TCHARs and 
  *  be able to compile for both unicode and non-unicode with the same source.
- *  This is not recommanded for linux applications and is not supported
+ *  This is not recommended for linux applications and is not supported
  * 	by the standard linux string header files.
  */
 #ifdef SQL_WCHART_CONVERT
 typedef wchar_t             TCHAR;
 #else
-typedef signed short        TCHAR;
+typedef unsigned short      TCHAR;
 #endif
 
 #else
 typedef char				TCHAR;
 #endif
 
-#ifndef DONT_TD_VOID
-typedef void				VOID;
-#endif
-
 typedef unsigned short		WORD;
 #if (SIZEOF_LONG_INT == 4)
 typedef unsigned long		DWORD;
@@ -114,7 +116,9 @@ typedef TCHAR*              LPTSTR;
 typedef char*               LPSTR;
 typedef DWORD*           	LPDWORD;
 
+#ifndef _WINDOWS_
 typedef void*               HINSTANCE;
+#endif
 
 #endif
 
@@ -132,12 +136,12 @@ typedef double          SQLFLOAT;
 #endif
 
 /*
- * can't use a long it fails on 64 platforms
+ * can't use a long; it fails on 64 platforms
  */
 
 /*
  * Hopefully by now it should be safe to assume most drivers know about SQLLEN now
- * and the defaukt is now sizeof( SQLLEN ) = 8 on 64 bit platforms
+ * and the default is now sizeof( SQLLEN ) = 8 on 64 bit platforms
  * 
  */
 
@@ -174,9 +178,19 @@ typedef unsigned long   SQLSETPOSIROW;
 #else
 typedef long            SQLINTEGER;
 typedef unsigned long   SQLUINTEGER;
+
+/* Handle case of building on mingw-w64 */
+
+#ifdef _WIN64
+typedef long long SQLLEN;
+typedef unsigned long long SQLULEN;
+typedef unsigned long long SQLSETPOSIROW;
+#else
 #define SQLLEN          SQLINTEGER
 #define SQLULEN         SQLUINTEGER
 #define SQLSETPOSIROW   SQLUSMALLINT
+#endif
+
 typedef SQLULEN         SQLROWCOUNT;
 typedef SQLULEN         SQLROWSETSIZE;
 typedef SQLULEN         SQLTRANSID;
@@ -241,7 +255,7 @@ typedef void *          	    HSTMT;
 /****************************
  * more basic data types to augment what windows.h provides
  ***************************/
-#ifndef ALLREADY_HAVE_WINDOWS_TYPE
+#ifndef ALREADY_HAVE_WINDOWS_TYPE
 
 typedef unsigned char           UCHAR;
 typedef signed char             SCHAR;
@@ -371,10 +385,14 @@ typedef struct tagSQL_INTERVAL_STRUCT
 # if (SIZEOF_LONG_INT == 8)
 #   define ODBCINT64	    long
 #   define UODBCINT64	unsigned long
+#   define ODBCINT64_TYPE	    "long"
+#   define UODBCINT64_TYPE	"unsigned long"
 # else
 #  ifdef HAVE_LONG_LONG
 #   define ODBCINT64	    long long
 #   define UODBCINT64	unsigned long long
+#   define ODBCINT64_TYPE	    "long long"
+#   define UODBCINT64_TYPE	"unsigned long long"
 #  else
 /*
  * may fail in some cases, but what else can we do ?
@@ -391,6 +409,8 @@ struct __bigint_struct_u
 };
 #   define ODBCINT64	    struct __bigint_struct
 #   define UODBCINT64	struct __bigint_struct_u
+#   define ODBCINT64_TYPE	    "struct __bigint_struct"
+#   define UODBCINT64_TYPE	"struct __bigint_struct_u"
 #  endif
 # endif
 #endif
@@ -420,7 +440,7 @@ typedef struct tagSQL_NUMERIC_STRUCT
 
 #if (ODBCVER >= 0x0350)
 #ifdef GUID_DEFINED
-#ifndef ALLREADY_HAVE_WINDOWS_TYPE
+#ifndef ALREADY_HAVE_WINDOWS_TYPE
 typedef GUID	SQLGUID;
 #else
 typedef struct  tagSQLGUID
