@@ -10,6 +10,11 @@
  * Peter Harvey		- pharvey@codebydesign.com
  **************************************************/
 #include <config.h>
+
+#ifdef UNIXODBC_SOURCE
+#include <ltdl.h>
+#endif
+
 #include <odbcinstext.h>
 
 int ODBCINSTDestructProperties( HODBCINSTPROPERTY *hFirstProperty )
