@@ -365,7 +365,7 @@ void __clear_ini_cache( void )
 
 #endif
 
-int SQLGetPrivateProfileString( LPCSTR  pszSection,
+int __SQLGetPrivateProfileStringNL( LPCSTR  pszSection,
                                 LPCSTR  pszEntry,
                                 LPCSTR  pszDefault,
                                 LPSTR   pRetBuffer,
@@ -605,6 +605,23 @@ int SQLGetPrivateProfileString( LPCSTR  pszSection,
     return ret;
 }
 
+int SQLGetPrivateProfileString( LPCSTR  pszSection,
+                                LPCSTR  pszEntry,
+                                LPCSTR  pszDefault,
+                                LPSTR   pRetBuffer,
+                                int     nRetBuffer,
+                                LPCSTR  pszFileName
+                              )
+{
+int ret;
+
+    __lock_config_mode();
+    ret = __SQLGetPrivateProfileStringNL( pszSection, pszEntry, pszDefault, pRetBuffer, nRetBuffer, pszFileName );
+    __unlock_config_mode();
+
+    return ret;
+}
+
 int  INSTAPI SQLGetPrivateProfileStringW( LPCWSTR lpszSection,
                                         LPCWSTR lpszEntry,
                                         LPCWSTR lpszDefault,
@@ -642,7 +659,7 @@ int  INSTAPI SQLGetPrivateProfileStringW( LPCWSTR lpszSection,
 		buf = NULL;
 	}
 
-	ret = SQLGetPrivateProfileString( sect, entry, def, buf, cbRetBuffer, name );
+	ret = buf ? SQLGetPrivateProfileString( sect, entry, def, buf, cbRetBuffer, name ) : -1;
 
 	if ( sect )
 		free( sect );
@@ -658,9 +675,9 @@ int  INSTAPI SQLGetPrivateProfileStringW( LPCWSTR lpszSection,
 		if ( buf && lpszRetBuffer )
 		{
             if ( !lpszSection || !lpszEntry )
-                _multi_string_copy_to_wide( lpszRetBuffer, buf, ret );
+                ret = _multi_string_copy_to_wide( lpszRetBuffer, buf, ret );
             else
-                _single_copy_to_wide( lpszRetBuffer, buf, ret );
+                ret = _single_copy_to_wide( lpszRetBuffer, buf, ret );
 		}
 	}
 
