@@ -15,6 +15,7 @@
  * Peter Harvey		- pharvey@codebydesign.com
  **************************************************/
 
+#include <config.h>
 #include <odbcinstext.h>
 
 BOOL _SQLWriteInstalledDrivers(
@@ -23,8 +24,8 @@ BOOL _SQLWriteInstalledDrivers(
 								LPCSTR	pszString )
 {
 	HINI	hIni;
-    char    szIniName[ INI_MAX_OBJECT_NAME + 1 ];
-	char	b1[ 256 ], b2[ 256 ];
+    char    szIniName[ ODBC_FILENAME_MAX * 2 + 1 ];
+	char	b1[ ODBC_FILENAME_MAX + 1 ], b2[ ODBC_FILENAME_MAX + 1 ];
 
 	/* SANITY CHECKS */
 	if ( pszSection == NULL )
